@@ -1,4 +1,3 @@
-import io
 import sys
 from abc import ABC, abstractmethod
 from collections import deque
@@ -8,7 +7,6 @@ from datetime import timedelta
 from math import ceil
 from threading import Event, RLock, Thread
 from typing import (
-    IO,
     Any,
     Callable,
     Deque,
@@ -25,23 +23,20 @@ from typing import (
 )
 
 from . import filesize, get_console
-from .ansi import AnsiDecoder
 from .console import (
     Console,
-    ConsoleRenderable,
     JustifyMethod,
     RenderableType,
     RenderGroup,
-    RenderHook,
 )
-from .control import Control
-from .highlighter import Highlighter
 from .jupyter import JupyterMixin
-from .live_render import LiveRender
+from .highlighter import Highlighter
+from .live import Live
 from .progress_bar import ProgressBar
+from .spinner import Spinner
 from .style import StyleType
-from .table import Table
-from .text import Text
+from .table import Column, Table
+from .text import Text, TextType
 
 TaskID = NewType("TaskID", int)
 
@@ -93,12 +88,13 @@ def track(
     console: Optional[Console] = None,
     transient: bool = False,
     get_time: Callable[[], float] = None,
-    refresh_per_second: int = None,
+    refresh_per_second: float = 10,
     style: StyleType = "bar.back",
     complete_style: StyleType = "bar.complete",
     finished_style: StyleType = "bar.finished",
     pulse_style: StyleType = "bar.pulse",
     update_period: float = 0.1,
+    disable: bool = False,
 ) -> Iterable[ProgressType]:
     """Track progress by iterating over a sequence.
 
@@ -109,12 +105,13 @@ def track(
         auto_refresh (bool, optional): Automatic refresh, disable to force a refresh after each iteration. Default is True.
         transient: (bool, optional): Clear the progress on exit. Defaults to False.
         console (Console, optional): Console to write to. Default creates internal Console instance.
-        refresh_per_second (Optional[int], optional): Number of times per second to refresh the progress information, or None to use default. Defaults to None.
+        refresh_per_second (float): Number of times per second to refresh the progress information. Defaults to 10.
         style (StyleType, optional): Style for the bar background. Defaults to "bar.back".
         complete_style (StyleType, optional): Style for the completed bar. Defaults to "bar.complete".
         finished_style (StyleType, optional): Style for a finished bar. Defaults to "bar.done".
         pulse_style (StyleType, optional): Style for pulsing bars. Defaults to "bar.pulse".
         update_period (float, optional): Minimum time (in seconds) between calls to update(). Defaults to 0.1.
+        disable (bool, optional): Disable display of progress.
     Returns:
         Iterable[ProgressType]: An iterable of the values in the sequence.
 
@@ -141,7 +138,8 @@ def track(
         console=console,
         transient=transient,
         get_time=get_time,
-        refresh_per_second=refresh_per_second,
+        refresh_per_second=refresh_per_second or 10,
+        disable=disable,
     )
 
     with progress:
@@ -155,10 +153,15 @@ class ProgressColumn(ABC):
 
     max_refresh: Optional[float] = None
 
-    def __init__(self) -> None:
+    def __init__(self, table_column: Column = None) -> None:
+        self._table_column = table_column
         self._renderable_cache: Dict[TaskID, Tuple[float, RenderableType]] = {}
         self._update_time: Optional[float] = None
 
+    def get_table_column(self) -> Column:
+        """Get a table column, used to build tasks table."""
+        return self._table_column or Column()
+
     def __call__(self, task: "Task") -> RenderableType:
         """Called by the Progress object to return a renderable for the given task.
 
@@ -187,6 +190,71 @@ class ProgressColumn(ABC):
         """Should return a renderable object."""
 
 
+class RenderableColumn(ProgressColumn):
+    """A column to insert an arbitrary column.
+
+    Args:
+        renderable (RenderableType, optional): Any renderable. Defaults to empty string.
+    """
+
+    def __init__(self, renderable: RenderableType = "", *, table_column: Column = None):
+        self.renderable = renderable
+        super().__init__(table_column=table_column)
+
+    def render(self, task: "Task") -> RenderableType:
+        return self.renderable
+
+
+class SpinnerColumn(ProgressColumn):
+    """A column with a 'spinner' animation.
+
+    Args:
+        spinner_name (str, optional): Name of spinner animation. Defaults to "dots".
+        style (StyleType, optional): Style of spinner. Defaults to "progress.spinner".
+        speed (float, optional): Speed factor of spinner. Defaults to 1.0.
+        finished_text (TextType, optional): Text used when task is finished. Defaults to " ".
+    """
+
+    def __init__(
+        self,
+        spinner_name: str = "dots",
+        style: Optional[StyleType] = "progress.spinner",
+        speed: float = 1.0,
+        finished_text: TextType = " ",
+        table_column: Column = None,
+    ):
+        self.spinner = Spinner(spinner_name, style=style, speed=speed)
+        self.finished_text = (
+            Text.from_markup(finished_text)
+            if isinstance(finished_text, str)
+            else finished_text
+        )
+        super().__init__(table_column=table_column)
+
+    def set_spinner(
+        self,
+        spinner_name: str,
+        spinner_style: Optional[StyleType] = "progress.spinner",
+        speed: float = 1.0,
+    ):
+        """Set a new spinner.
+
+        Args:
+            spinner_name (str): Spinner name, see python -m rich.spinner.
+            spinner_style (Optional[StyleType], optional): Spinner style. Defaults to "progress.spinner".
+            speed (float, optional): Speed factor of spinner. Defaults to 1.0.
+        """
+        self.spinner = Spinner(spinner_name, style=spinner_style, speed=speed)
+
+    def render(self, task: "Task") -> Text:
+        text = (
+            self.finished_text
+            if task.finished
+            else self.spinner.render(task.get_time())
+        )
+        return text
+
+
 class TextColumn(ProgressColumn):
     """A column containing text."""
 
@@ -197,13 +265,14 @@ class TextColumn(ProgressColumn):
         justify: JustifyMethod = "left",
         markup: bool = True,
         highlighter: Highlighter = None,
+        table_column: Column = None,
     ) -> None:
         self.text_format = text_format
         self.justify = justify
         self.style = style
         self.markup = markup
         self.highlighter = highlighter
-        super().__init__()
+        super().__init__(table_column=table_column or Column(no_wrap=True))
 
     def render(self, task: "Task") -> Text:
         _text = self.text_format.format(task=task)
@@ -234,13 +303,14 @@ class BarColumn(ProgressColumn):
         complete_style: StyleType = "bar.complete",
         finished_style: StyleType = "bar.finished",
         pulse_style: StyleType = "bar.pulse",
+        table_column: Column = None,
     ) -> None:
         self.bar_width = bar_width
         self.style = style
         self.complete_style = complete_style
         self.finished_style = finished_style
         self.pulse_style = pulse_style
-        super().__init__()
+        super().__init__(table_column=table_column)
 
     def render(self, task: "Task") -> ProgressBar:
         """Gets a progress bar widget for a task."""
@@ -257,6 +327,18 @@ class BarColumn(ProgressColumn):
         )
 
 
+class TimeElapsedColumn(ProgressColumn):
+    """Renders time elapsed."""
+
+    def render(self, task: "Task") -> Text:
+        """Show time remaining."""
+        elapsed = task.finished_time if task.finished else task.elapsed
+        if elapsed is None:
+            return Text("-:--:--", style="progress.elapsed")
+        delta = timedelta(seconds=int(elapsed))
+        return Text(str(delta), style="progress.elapsed")
+
+
 class TimeRemainingColumn(ProgressColumn):
     """Renders estimated time remaining."""
 
@@ -297,9 +379,9 @@ class DownloadColumn(ProgressColumn):
         binary_units (bool, optional): Use binary units, KiB, MiB etc. Defaults to False.
     """
 
-    def __init__(self, binary_units: bool = False) -> None:
+    def __init__(self, binary_units: bool = False, table_column: Column = None) -> None:
         self.binary_units = binary_units
-        super().__init__()
+        super().__init__(table_column=table_column)
 
     def render(self, task: "Task") -> Text:
         """Calculate common unit for completed and total."""
@@ -330,7 +412,7 @@ class TransferSpeedColumn(ProgressColumn):
 
     def render(self, task: "Task") -> Text:
         """Show data transfer speed."""
-        speed = task.speed
+        speed = task.finished_speed or task.speed
         if speed is None:
             return Text("?", style="progress.data.speed")
         data_speed = filesize.decimal(int(speed))
@@ -369,6 +451,9 @@ class Task:
     _get_time: GetTimeCallable
     """Callable to get the current time."""
 
+    finished_time: Optional[float] = None
+    """float: Time task was finished."""
+
     visible: bool = True
     """bool: Indicates if this task is visible in the progress display."""
 
@@ -381,10 +466,16 @@ class Task:
     stop_time: Optional[float] = field(default=None, init=False, repr=False)
     """Optional[float]: Time this task was stopped, or None if not stopped."""
 
+    finished_speed: Optional[float] = None
+    """Optional[float]: The last speed for a finshed task."""
+
     _progress: Deque[ProgressSample] = field(
         default_factory=deque, init=False, repr=False
     )
 
+    _lock: RLock = field(repr=False, default_factory=RLock)
+    """Thread lock."""
+
     def get_time(self) -> float:
         """float: Get the current time, in seconds."""
         return self._get_time()  # type: ignore
@@ -410,8 +501,8 @@ class Task:
 
     @property
     def finished(self) -> bool:
-        """bool: Check if the task has completed."""
-        return self.completed >= self.total
+        """Check if the task has finished."""
+        return self.finished_time is not None
 
     @property
     def percentage(self) -> float:
@@ -427,17 +518,18 @@ class Task:
         """Optional[float]: Get the estimated speed in steps per second."""
         if self.start_time is None:
             return None
-        progress = self._progress
-        if not progress:
-            return None
-        total_time = progress[-1].timestamp - progress[0].timestamp
-        if total_time == 0:
-            return None
-        iter_progress = iter(progress)
-        next(iter_progress)
-        total_completed = sum(sample.completed for sample in iter_progress)
-        speed = total_completed / total_time
-        return speed
+        with self._lock:
+            progress = self._progress
+            if not progress:
+                return None
+            total_time = progress[-1].timestamp - progress[0].timestamp
+            if total_time == 0:
+                return None
+            iter_progress = iter(progress)
+            next(iter_progress)
+            total_completed = sum(sample.completed for sample in iter_progress)
+            speed = total_completed / total_time
+            return speed
 
     @property
     def time_remaining(self) -> Optional[float]:
@@ -453,77 +545,24 @@ class Task:
     def _reset(self) -> None:
         """Reset progress."""
         self._progress.clear()
+        self.finished_time = None
+        self.finished_speed = None
 
 
-class _RefreshThread(Thread):
-    """A thread that calls refresh() on the Process object at regular intervals."""
-
-    def __init__(self, progress: "Progress", refresh_per_second: int = 10) -> None:
-        self.progress = progress
-        self.refresh_per_second = refresh_per_second
-        self.done = Event()
-        super().__init__()
-
-    def stop(self) -> None:
-        self.done.set()
-
-    def run(self) -> None:
-        while not self.done.wait(1.0 / self.refresh_per_second):
-            self.progress.refresh()
-
-
-class _FileProxy(io.TextIOBase):
-    """Wraps a file (e.g. sys.stdout) and redirects writes to a console."""
-
-    def __init__(self, console: Console, file: IO[str]) -> None:
-        self.__console = console
-        self.__file = file
-        self.__buffer: List[str] = []
-        self.__ansi_decoder = AnsiDecoder()
-
-    def __getattr__(self, name: str) -> Any:
-        return getattr(self.__file, name)
-
-    def write(self, text: str) -> int:
-        buffer = self.__buffer
-        lines: List[str] = []
-        while text:
-            line, new_line, text = text.partition("\n")
-            if new_line:
-                lines.append("".join(buffer) + line)
-                del buffer[:]
-            else:
-                buffer.append(line)
-                break
-        if lines:
-            console = self.__console
-            with console:
-                output = Text("\n").join(
-                    self.__ansi_decoder.decode_line(line) for line in lines
-                )
-                console.print(output, markup=False, emoji=False, highlight=False)
-        return len(text)
-
-    def flush(self) -> None:
-        buffer = self.__buffer
-        if buffer:
-            self.__console.print("".join(buffer))
-            del buffer[:]
-
-
-class Progress(JupyterMixin, RenderHook):
+class Progress(JupyterMixin):
     """Renders an auto-updating progress bar(s).
 
     Args:
         console (Console, optional): Optional Console instance. Default will an internal Console instance writing to stdout.
         auto_refresh (bool, optional): Enable auto refresh. If disabled, you will need to call `refresh()`.
-        refresh_per_second (Optional[int], optional): Number of times per second to refresh the progress information or None to use default (10). Defaults to None.
+        refresh_per_second (Optional[float], optional): Number of times per second to refresh the progress information or None to use default (10). Defaults to None.
         speed_estimate_period: (float, optional): Period (in seconds) used to calculate the speed estimate. Defaults to 30.
         transient: (bool, optional): Clear the progress on exit. Defaults to False.
         redirect_stdout: (bool, optional): Enable redirection of stdout, so ``print`` may be used. Defaults to True.
         redirect_stderr: (bool, optional): Enable redirection of stderr. Defaults to True.
         get_time: (Callable, optional): A callable that gets the current time, or None to use Console.get_time. Defaults to None.
         disable (bool, optional): Disable progress display. Defaults to False
+        expand (bool, optional): Expand tasks table to fit width. Defaults to False.
     """
 
     def __init__(
@@ -531,16 +570,17 @@ class Progress(JupyterMixin, RenderHook):
         *columns: Union[str, ProgressColumn],
         console: Console = None,
         auto_refresh: bool = True,
-        refresh_per_second: int = None,
+        refresh_per_second: float = 10,
         speed_estimate_period: float = 30.0,
         transient: bool = False,
         redirect_stdout: bool = True,
         redirect_stderr: bool = True,
         get_time: GetTimeCallable = None,
         disable: bool = False,
+        expand: bool = False,
     ) -> None:
         assert (
-            refresh_per_second is None or refresh_per_second > 0
+            refresh_per_second is None or refresh_per_second > 0  # type: ignore
         ), "refresh_per_second must be > 0"
         self._lock = RLock()
         self.columns = columns or (
@@ -549,25 +589,28 @@ class Progress(JupyterMixin, RenderHook):
             TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
             TimeRemainingColumn(),
         )
-        self.console = console or get_console()
-        self.auto_refresh = auto_refresh and not self.console.is_jupyter
-        self.refresh_per_second = refresh_per_second or 10
         self.speed_estimate_period = speed_estimate_period
-        self.transient = transient
-        self._redirect_stdout = redirect_stdout
-        self._redirect_stderr = redirect_stderr
-        self.get_time = get_time or self.console.get_time
+
         self.disable = disable
+        self.expand = expand
         self._tasks: Dict[TaskID, Task] = {}
-        self._live_render = LiveRender(self.get_renderable())
         self._task_index: TaskID = TaskID(0)
-        self._refresh_thread: Optional[_RefreshThread] = None
-        self._started = False
+        self.live = Live(
+            console=console or get_console(),
+            auto_refresh=auto_refresh,
+            refresh_per_second=refresh_per_second,
+            transient=transient,
+            redirect_stdout=redirect_stdout,
+            redirect_stderr=redirect_stderr,
+            get_renderable=self.get_renderable,
+        )
+        self.get_time = get_time or self.console.get_time
         self.print = self.console.print
         self.log = self.console.log
-        self._restore_stdout: Optional[IO[str]] = None
-        self._restore_stderr: Optional[IO[str]] = None
-        self.ipy_widget: Optional[Any] = None
+
+    @property
+    def console(self) -> Console:
+        return self.live.console
 
     @property
     def tasks(self) -> List[Task]:
@@ -589,63 +632,13 @@ class Progress(JupyterMixin, RenderHook):
                 return True
             return all(task.finished for task in self._tasks.values())
 
-    def _enable_redirect_io(self):
-        """Enable redirecting of stdout / stderr."""
-        if self.console.is_terminal:
-            if self._redirect_stdout:
-                self._restore_stdout = sys.stdout
-                sys.stdout = _FileProxy(self.console, sys.stdout)
-            if self._redirect_stderr:
-                self._restore_stderr = sys.stderr
-                sys.stderr = _FileProxy(self.console, sys.stderr)
-
-    def _disable_redirect_io(self):
-        """Disable redirecting of stdout / stderr."""
-        if self._restore_stdout:
-            sys.stdout = self._restore_stdout
-            self._restore_stdout = None
-        if self._restore_stderr:
-            sys.stderr = self._restore_stderr
-            self._restore_stderr = None
-
     def start(self) -> None:
         """Start the progress display."""
-        with self._lock:
-            if self._started:
-                return
-            self._started = True
-            self.console.show_cursor(False)
-            self._enable_redirect_io()
-            self.console.push_render_hook(self)
-            self.refresh()
-            if self.auto_refresh:
-                self._refresh_thread = _RefreshThread(self, self.refresh_per_second)
-                self._refresh_thread.start()
+        self.live.start(refresh=True)
 
     def stop(self) -> None:
         """Stop the progress display."""
-        with self._lock:
-            if not self._started:
-                return
-            self._started = False
-            try:
-                if self.auto_refresh and self._refresh_thread is not None:
-                    self._refresh_thread.stop()
-                self.refresh()
-                if self.console.is_terminal:
-                    self.console.line()
-            finally:
-                self.console.show_cursor(True)
-                self._disable_redirect_io()
-                self.console.pop_render_hook()
-        if self._refresh_thread is not None:
-            self._refresh_thread.join()
-            self._refresh_thread = None
-        if self.transient:
-            self.console.control(self._live_render.restore_cursor())
-        if self.ipy_widget is not None and self.transient:  # pragma: no cover
-            self.ipy_widget.clear_output()
-            self.ipy_widget.close()
+        self.live.stop()
 
     def __enter__(self) -> "Progress":
         self.start()
@@ -674,6 +667,7 @@ class Progress(JupyterMixin, RenderHook):
         Returns:
             Iterable[ProgressType]: An iterable of values taken from the provided sequence.
         """
+
         if total is None:
             if isinstance(sequence, Sized):
                 task_total = len(sequence)
@@ -689,7 +683,7 @@ class Progress(JupyterMixin, RenderHook):
         else:
             self.update(task_id, total=task_total)
 
-        if self.auto_refresh:
+        if self.live.auto_refresh:
             with _TrackThread(self, task_id, update_period) as track_thread:
                 for value in sequence:
                     yield value
@@ -773,9 +767,6 @@ class Progress(JupyterMixin, RenderHook):
             task.fields.update(fields)
             update_completed = task.completed - completed_start
 
-            if refresh:
-                self.refresh()
-
             current_time = self.get_time()
             old_sample_time = current_time - self.speed_estimate_period
             _progress = task._progress
@@ -787,6 +778,11 @@ class Progress(JupyterMixin, RenderHook):
                 popleft()
             if update_completed > 0:
                 _progress.append(ProgressSample(current_time, update_completed))
+            if task.completed >= task.total and task.finished_time is None:
+                task.finished_time = task.elapsed
+
+        if refresh:
+            self.refresh()
 
     def reset(
         self,
@@ -822,7 +818,8 @@ class Progress(JupyterMixin, RenderHook):
                 task.fields = fields
             if description is not None:
                 task.description = description
-            self.refresh()
+            task.finished_time = None
+        self.refresh()
 
     def advance(self, task_id: TaskID, advance: float = 1) -> None:
         """Advance task by a number of steps.
@@ -846,36 +843,14 @@ class Progress(JupyterMixin, RenderHook):
             while len(_progress) > 1000:
                 popleft()
             _progress.append(ProgressSample(current_time, update_completed))
+            if task.completed >= task.total and task.finished_time is None:
+                task.finished_time = task.elapsed
+                task.finished_speed = task.speed
 
     def refresh(self) -> None:
         """Refresh (render) the progress information."""
-        if self.console.is_jupyter:  # pragma: no cover
-            try:
-                from IPython.display import display
-                from ipywidgets import Output
-            except ImportError:
-                import warnings
-
-                warnings.warn('install "ipywidgets" for Jupyter support')
-            else:
-                with self._lock:
-                    if self.ipy_widget is None:
-                        self.ipy_widget = Output()
-                        display(self.ipy_widget)
-
-                    with self.ipy_widget:
-                        self.ipy_widget.clear_output(wait=True)
-                        self.console.print(self.get_renderable())
-
-        elif (
-            self.console.is_terminal
-            and not self.console.is_dumb_terminal
-            and not self.disable
-        ):
-            with self._lock:
-                self._live_render.set_renderable(self.get_renderable())
-                with self.console:
-                    self.console.print(Control(""))
+        if not self.disable:
+            self.live.refresh()
 
     def get_renderable(self) -> RenderableType:
         """Get a renderable for the progress display."""
@@ -897,25 +872,34 @@ class Progress(JupyterMixin, RenderHook):
             Table: A table instance.
         """
 
-        table = Table.grid(padding=(0, 1))
-        for _ in self.columns:
-            table.add_column()
+        table_columns = (
+            (
+                Column(no_wrap=True)
+                if isinstance(_column, str)
+                else _column.get_table_column().copy()
+            )
+            for _column in self.columns
+        )
+        table = Table.grid(*table_columns, padding=(0, 1), expand=self.expand)
+
         for task in tasks:
             if task.visible:
-                row: List[RenderableType] = []
-                append = row.append
-                for index, column in enumerate(self.columns):
-                    if isinstance(column, str):
-                        append(column.format(task=task))
-                        table.columns[index].no_wrap = True
-                    else:
-                        widget = column(task)
-                        append(widget)
-                        if isinstance(widget, (str, Text)):
-                            table.columns[index].no_wrap = True
-                table.add_row(*row)
+                table.add_row(
+                    *(
+                        (
+                            column.format(task=task)
+                            if isinstance(column, str)
+                            else column(task)
+                        )
+                        for column in self.columns
+                    )
+                )
         return table
 
+    def __rich__(self) -> RenderableType:
+        """Makes the Progress class itself renderable."""
+        return self.get_renderable()
+
     def add_task(
         self,
         description: str,
@@ -948,15 +932,15 @@ class Progress(JupyterMixin, RenderHook):
                 visible=visible,
                 fields=fields,
                 _get_time=self.get_time,
+                _lock=self._lock,
             )
             self._tasks[self._task_index] = task
             if start:
                 self.start_task(self._task_index)
-            self.refresh()
-            try:
-                return self._task_index
-            finally:
-                self._task_index = TaskID(int(self._task_index) + 1)
+            new_task_index = self._task_index
+            self._task_index = TaskID(int(self._task_index) + 1)
+        self.refresh()
+        return new_task_index
 
     def remove_task(self, task_id: TaskID) -> None:
         """Delete a task if it exists.
@@ -968,18 +952,6 @@ class Progress(JupyterMixin, RenderHook):
         with self._lock:
             del self._tasks[task_id]
 
-    def process_renderables(
-        self, renderables: List[ConsoleRenderable]
-    ) -> List[ConsoleRenderable]:
-        """Process renderables to restore cursor and display progress."""
-        if self.console.is_terminal:
-            renderables = [
-                self._live_render.position_cursor(),
-                *renderables,
-                self._live_render,
-            ]
-        return renderables
-
 
 if __name__ == "__main__":  # pragma: no coverage
 
@@ -1027,19 +999,25 @@ if __name__ == "__main__":  # pragma: no coverage
     examples = cycle(progress_renderables)
 
     console = Console(record=True)
-    try:
-        with Progress(console=console, transient=True) as progress:
 
-            task1 = progress.add_task("[red]Downloading", total=1000)
-            task2 = progress.add_task("[green]Processing", total=1000)
-            task3 = progress.add_task("[yellow]Thinking", total=1000, start=False)
+    with Progress(
+        SpinnerColumn(),
+        TextColumn("[progress.description]{task.description}"),
+        BarColumn(),
+        TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
+        TimeRemainingColumn(),
+        TimeElapsedColumn(),
+        console=console,
+        transient=True,
+    ) as progress:
 
-            while not progress.finished:
-                progress.update(task1, advance=0.5)
-                progress.update(task2, advance=0.3)
-                time.sleep(0.01)
-                if random.randint(0, 100) < 1:
-                    progress.log(next(examples))
-    except:
-        console.save_html("progress.html")
-        print("wrote progress.html")
+        task1 = progress.add_task("[red]Downloading", total=1000)
+        task2 = progress.add_task("[green]Processing", total=1000)
+        task3 = progress.add_task("[yellow]Thinking", total=1000, start=False)
+
+        while not progress.finished:
+            progress.update(task1, advance=0.5)
+            progress.update(task2, advance=0.3)
+            time.sleep(0.01)
+            if random.randint(0, 100) < 1:
+                progress.log(next(examples))
